﻿using System;
using System.Data.SqlClient;
using System.Linq;
using JLHHJSvr.BLL;
using JLHHJSvr.Com;
using JLHHJSvr.LJException;
using LJLib.DAL.SQL;
using LJLib.Net.SPI.Server;

namespace JLHHJSvr.Excutor
{
    internal sealed class DeleteMtrlDefExcutor : ExcutorBase<DeleteMtrlDefRequest, DeleteMtrlDefResponse>
    {
        protected override void ExcuteInternal(DeleteMtrlDefRequest request, object state, DeleteMtrlDefResponse rslt)
        {
            var tokendata = BllHelper.GetToken(request.token);
            if (tokendata == null)
            {
                rslt.ErrMsg = "会话已经中断，请重新登录";
                return;
            }
            if (!request.list.Any())
            {
                rslt.ErrMsg = "至少提交一条需要删除的记录";
                return;
            }

            using (var con = new SqlConnection(GlobalVar.ConnectionString))
            using (var cmd = con.CreateCommand())
            {
                con.Open();

                using (cmd.Transaction = con.BeginTransaction())
                {
                    try
                    {
                        foreach (var mtrl in request.list)
                        {
                            // 删除物料价格明细
                            cmd.CommandText = @"DELETE FROM u_mtrl_price_pricelist WHERE mtrlid = @mtrlid";
                            cmd.Parameters.Clear();
                            cmd.Parameters.AddWithValue("@mtrlid", mtrl.mtrlid.Value);
                            cmd.ExecuteNonQuery();

                            if (DbSqlHelper.Delete(cmd, mtrl) == 0)
                            {
                                throw new LJCommonException($"物料类别不存在");
                            }
                        }

                        cmd.Transaction.Commit();
                    }
                    catch (Exception e)
                    {
                        cmd.Transaction.Rollback();
                        rslt.ErrMsg = e.ToString();
                    }
                }
            }
        }
    }
}
