﻿#region License, Terms and Conditions
//
// Jayrock - JSON and JSON-RPC for Microsoft .NET Framework and Mono
// Written by Atif Aziz (atif.aziz@skybow.com)
// Copyright (c) 2005 Atif Aziz. All rights reserved.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation; either version 2.1 of the License, or (at your option)
// any later version.
//
// This library is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
//
#endregion

using System.Collections.Generic;

namespace Jayrock.Json.Conversion.Converters
{
    #region Imports

    using System;
    using System.Diagnostics;

    #endregion

    internal sealed class ByteArrayExporter : ExporterBase
    {
        /// <summary>
        /// 上下文前缀关键字
        /// </summary>
        public static readonly string Prefix = "ByteArrayExporter_Prefix";

        public ByteArrayExporter() : base(typeof(byte[])) { }

        protected override void ExportValue(ExportContext context, object value, JsonWriter writer)
        {
            Debug.Assert(context != null);
            Debug.Assert(value != null);
            Debug.Assert(writer != null);

            byte[] bytes = (byte[]) value;

            IDictionary<string, byte[]> files = null;
            var key = typeof (ByteArrayExporter);
            if (context.Items.Contains(key))
            {
                files = context.Items[key] as IDictionary<string, byte[]>;
            }
            var prefix = string.Empty;
            if (context.Items.Contains(Prefix))
            {
                prefix = context.Items[Prefix].ToString();
            }
            if (files != null)
            {
                string filekey = null;
                using (System.Security.Cryptography.MD5 md5 = System.Security.Cryptography.MD5.Create())
                {
                    filekey = Convert.ToBase64String(md5.ComputeHash(bytes))
                        .Replace('+', '_')
                        .Replace('/', '-')
                        .Replace("=", string.Empty);
                }
                if (!files.ContainsKey(filekey))
                {
                    files.Add(filekey, bytes);
                }
                writer.WriteString(prefix + filekey);
            }
            else
            {
                writer.WriteString(Convert.ToBase64String(bytes));
            }
        }
    }
}