import { ref, reactive, computed, toRefs, nextTick } from "vue";
import { Table } from "@/hooks/interface";
import { ColumnProps } from "@/components/LjVxeTable/interface";
import { ALLOW_EDIT_STATE } from "@/config/index";
import { GetMattressInterfaceList, RefreshMattressInterface, RefreshMattressInterfaceQd } from "@/api/modules/quote";
import { ElMessage, ElMessageBox } from "element-plus";
import { CommonDynamicSelect, GetERPMtrldefList, GetERPWrkGrpList, GetERPMtrlTypeList } from "@/api/modules/common";
import { transformTreeData } from "@/utils/index";
import { floatSub, floatAdd, floatMul, floatDiv, formatFixedNumber } from "@/utils/index";

interface defaultState {
  /**
   * @description 单据当前状态
   */
  orderStatus: string;
  /**
   * @description 列表Ref
   */
  VxeTableRef: any;
  /**
   * @description 详情页Ref
   */
  LjDetailRef: any;
  mainData: any;
  mattressYWList: any[];
  mattressQDList: any[];
  editType: number;
  MattressDialogRef: any;
  MattressDialogProps: any;
}

const state = reactive<defaultState>({
  orderStatus: "",
  VxeTableRef: null,
  LjDetailRef: null,
  mainData: {},
  mattressYWList: [],
  mattressQDList: [],
  editType: 0,
  MattressDialogRef: null,
  MattressDialogProps: {}
});
/**
 * @description 表格多选数据操作
 * @param {String} rowKey 当表格可以多选时，所指定的 id
 * */
export const useHooks = (t?: any) => {
  // 表格配置项
  const columns: ColumnProps<any>[] = [
    { type: "checkbox", width: 80, fixed: "left" },
    { title: "#", field: "pid", fixed: "left", width: 80 },
    {
      field: "yw_flag",
      title: "业务审核"
    },
    {
      field: "js1_flag",
      title: "产品补充审核"
    },
    {
      field: "js2_flag",
      title: "清单补充审核"
    },
    {
      field: "creatmtrl_flag",
      title: "生成L1产品"
    },
    {
      field: "creatmtrlqd_flag",
      title: "生成L1清单"
    },
    {
      field: "erp_mtrlcode",
      title: "L1物料编码"
    },
    {
      field: "deptid",
      title: "部门",
      search: {
        el: "select",
        key: "arg_deptid",
        props: {
          filterable: true
        },
        order: 1
      }
    },
    {
      field: "mattresscode",
      title: "报价唯一码"
    },
    {
      field: "mattressrelcode",
      title: "床垫编码"
    },
    {
      field: "mattressname",
      title: "床垫名称"
    },
    {
      field: "mattresstypeid",
      title: "床垫类别"
    },
    {
      field: "mattress_width",
      title: "床垫宽/CM"
    },
    {
      field: "mattress_length",
      title: "床垫长/CM"
    },
    {
      field: "mattress_height",
      title: "床垫高/CM"
    },
    {
      field: "createby",
      title: "登记人"
    },
    {
      field: "createtime",
      title: "登记时间"
    },
    {
      field: "yw_auditingrep",
      title: "业务审核人"
    },
    {
      field: "yw_auditingdate",
      title: "业务审核时间"
    },
    {
      field: "js1_auditingrep",
      title: "产品补充审核人"
    },
    {
      field: "js1_auditingdate",
      title: "产品补充审核时间"
    },
    {
      field: "js2_auditingrep",
      title: "清单补充审核人"
    },
    {
      field: "js2_auditingdate",
      title: "清单补充审核时间"
    },
    {
      field: "creatmtrl_auditingrep",
      title: "生成L1产品人"
    },
    {
      field: "creatmtrl_auditingdate",
      title: "生成L1产品时间"
    },
    {
      field: "creatmtrlqd_auditingrep",
      title: "生成L1清单人"
    },
    {
      field: "creatmtrlqd_auditingdate",
      title: "生成L1清单时间"
    },
    {
      field: "dijia_cost",
      title: "底价"
    },
    {
      field: "yongjin",
      title: "佣金",
      render: (scope: any) => {
        let result = floatSub(floatDiv(scope.row.dijia_cost, floatSub(1, scope.row.commission - 1)), scope.row.dijia_cost);
        result = formatFixedNumber({ val: result });
        return <>{result}</>;
      }
    },
    {
      field: "taxrate",
      title: "税率"
    },
    {
      field: "fob",
      title: "FOB"
    },
    {
      field: "taxes",
      title: "税金"
    },
    {
      field: "erp_mtrlengname",
      title: "原物料编码"
    },
    {
      field: "erp_configcodetype",
      title: "ERP配置类型"
    }
  ];

  const columns_detail: ColumnProps<any>[] = [
    {
      field: "erp_mtrlcode",
      title: "新物料编码",
      basicinfo: {
        el: "input",
        editable: () => {
          return false;
        },
        group: "单据信息",
        order: 1,
        span: 1
      }
    },
    {
      field: "erp_mtrlname",
      title: "新物料名称",
      basicinfo: {
        el: "input",
        editable: () => {
          return false;
        },
        group: "单据信息",
        order: 4,
        span: 1
      }
    },
    {
      field: "erp_mtrlmode",
      title: "新物料规格",
      basicinfo: {
        el: "input",
        editable: ALLOW_EDIT_STATE,
        group: "单据信息",
        order: 2,
        span: 1
      }
    },
    {
      field: "erp_mtrlunit",
      title: "物料单位",
      basicinfo: {
        el: "input",
        editable: ALLOW_EDIT_STATE,
        group: "单据信息",
        order: 5,
        span: 1
      }
    },
    {
      field: "erp_mtrltype",
      title: "物料类别",
      basicinfo: {
        el: "tree-select",
        editable: ALLOW_EDIT_STATE,
        group: "单据信息",
        order: 3,
        span: 1
      },
      enum: async () => {
        const data = await GetERPMtrlTypeList();
        return transformTreeData(data.reList);
      },
      fieldNames: {
        label: "mtrltype",
        value: "mtrltypeid",
        children: "children"
      }
    },
    {
      field: "erp_configcodetype",
      title: "配置类型",
      basicinfo: {
        el: "input",
        editable: ALLOW_EDIT_STATE,
        group: "单据信息",
        order: 6,
        span: 1
      }
    },
    {
      field: "mattresscode",
      title: "图号(床垫唯一码)",
      basicinfo: {
        el: "input",
        editable: () => {
          return false;
        },
        group: "单据信息",
        order: 8,
        span: 1
      }
    },
    {
      field: "erp_mtrlengname",
      title: "英文名称",
      basicinfo: {
        el: "input",
        editable: ALLOW_EDIT_STATE,
        group: "单据信息",
        order: 7,
        span: 1
      }
    },
    {
      field: "old_mtrlname",
      title: "旧物料名称",
      basicinfo: {
        el: "input",
        editable: ALLOW_EDIT_STATE,
        group: "单据信息",
        order: 9,
        span: 1
      }
    }
  ];

  const columns_yw: ColumnProps<any>[] = [
    {
      field: "printid",
      title: "排序",
      width: 80
    },
    {
      field: "itemname",
      title: "项目"
    },
    {
      field: "bj_pzname",
      title: "核价_配置名称"
    },
    {
      field: "erp_pzcode",
      title: "ERP配置编码"
    },
    {
      field: "erp_pzname",
      title: "ERP配置名称"
    },
    {
      field: "bj_inputtype",
      title: "录入类型",
      enum: [
        {
          value: "0",
          label: "选择"
        },
        {
          value: "1",
          label: "录入"
        },
        {
          value: "2",
          label: "带出"
        }
      ],
      editRender: {
        name: "$select"
      }
    },
    {
      field: "bj_namemx",
      title: "明细名称",
      editRender: {
        name: "$input"
      }
    },
    {
      field: "actual_size",
      title: "生产尺寸",
      limited: () => {
        return state.editType == 1;
      },
      editRender: {
        name: "$input"
      }
    },
    {
      field: "sb_craft",
      title: "锁边工艺",
      limited: () => {
        return state.editType == 1;
      },
      editRender: {
        name: "$input"
      }
    },
    {
      field: "actual_size_sb",
      title: "锁边后尺寸",
      limited: () => {
        return state.editType == 1;
      },
      editRender: {
        name: "$input"
      }
    },
    {
      field: "ss_rate",
      title: "收缩率",
      limited: () => {
        return state.editType == 1;
      },
      datatype: "number",
      editRender: {
        name: "$input"
      }
    },
    {
      field: "ls_rate",
      title: "拉伸率",
      limited: () => {
        return state.editType == 1;
      },
      datatype: "number",
      editRender: {
        name: "$input"
      }
    }
  ];

  const columns_qd: ColumnProps<any>[] = [
    {
      field: "printid",
      title: "排序",
      width: 80
    },
    {
      field: "itemname",
      title: "项目"
    },
    {
      field: "bj_pzname",
      title: "二级项目"
    },
    {
      field: "bj_pzname_mx",
      title: "明细项目"
    },
    {
      field: "bj_pzname_mx_mx",
      title: "二级明细项目"
    },
    {
      field: "erp_mtrlcode",
      title: "L1物料编码",
      limited: () => {
        return state.editType == 1 || state.editType == 2;
      },
      editRender: {
        name: "$input"
      }
    },
    {
      field: "mtrlengname",
      title: "自定义2",
      editRender: {
        name: "$input"
      }
    },
    {
      field: "erp_mtrlname",
      title: "L1物料名称",
      limited: () => {
        return state.editType == 1 || state.editType == 2;
      }
    },
    {
      field: "erp_mtrlmode",
      title: "L1物料规格",
      limited: () => {
        return state.editType == 1 || state.editType == 2;
      }
    },
    {
      field: "erp_unit",
      title: "L1物料单位",
      limited: () => {
        return state.editType == 1 || state.editType == 2;
      }
    },
    {
      field: "useqty",
      title: "参考用料量",
      datatype: "number",
      limited: () => {
        return state.editType == 1 || state.editType == 2;
      }
    },
    {
      field: "actual_useqty",
      title: "实际用量",
      datatype: "number",
      editRender: {
        name: "$input"
      },
      limited: () => {
        return state.editType == 1 || state.editType == 2;
      }
    },
    {
      field: "wrkgrpid",
      title: "领用工组",
      enum: async () => {
        const data = await GetERPWrkGrpList();
        return { data: data.workgroupList };
      },
      isFilterEnum: true,
      fieldNames: { label: "wrkgrpname", value: "wrkgrpid" },
      editRender: {
        name: "$select"
      }
    },
    {
      field: "qd_actual_size",
      title: "实际尺寸",
      editRender: {
        name: "$input"
      }
    },
    {
      field: "qd_pfgroupqty",
      title: "条数",
      editRender: {
        name: "$input"
      }
    },
    {
      field: "dscrp",
      title: "备注",
      editRender: {
        name: "$input"
      }
    },
    {
      field: "ss_rate",
      title: "收缩率",
      datatype: "number",
      editRender: {
        name: "$input"
      }
    },
    {
      field: "ls_rate",
      title: "拉伸率",
      datatype: "number",
      editRender: {
        name: "$input"
      }
    },
    {
      field: "sh_rate",
      title: "损耗率",
      datatype: "number",
      editRender: {
        name: "$input"
      }
    }
  ];

  /**
   * 获取产品配置，产品清单
   * @param mattressid 床垫ID
   */
  const RetriveMattressInterface = async (mattressid: number) => {
    let res = await GetMattressInterfaceList({ mattressid });
    state.mainData = res.mattress;
    state.mattressYWList = res.interfaceList;
    state.mattressQDList = res.qdList;
  };
  /**
   * 获取产品配置，产品清单
   * @param mattressid 床垫ID
   */
  const RefreshMattressInterfaceList = async (mattressid: number, isPz: number) => {
    let res = await RefreshMattressInterface({ mattressid, isPz });
    state.mattressYWList = res.mxList;
  };
  /**
   * 获取产品配置，产品清单
   * @param mattressid 床垫ID
   */
  const RefreshMattressInterfaceQdList = async (mattressid: number, isPz: number, isCheck?: number) => {
    isCheck = isCheck ?? 1;

    let res = await RefreshMattressInterfaceQd({ mattressid, isPz, isCheck });

    const interfaceMap = {};
    state.mattressYWList.forEach(mb => {
      interfaceMap[mb.bj_pzname] = mb;
    });

    // 遍历 qdList 并使用哈希表来查找和赋值
    res.mxList.forEach(mx => {
      const mb = interfaceMap[mx.bj_pzname];
      if (mb) {
        mx.bj_pzname_mx_mx = mb.bj_namemx;

        if (mx.ss_rate === 0) {
          mx.ss_rate = mb.ss_rate;
        }

        if (mx.ls_rate === 0) {
          mx.ls_rate = mb.ls_rate;
        }
      }
    });

    state.mattressQDList = res.mxList;
  };

  /**
   * @description 弹窗模块：床垫选择
   * @param data 当前数据
   * @param status 当前订单状态
   * @returns Promise
   */
  const fModelChoseMattress = (label?: string) => {
    return new Promise((resolve, reject) => {
      let _params = {
        arg_deptid: state.mainData.deptid
      };
      state.MattressDialogProps = {
        onSubmit: (res: any) => {
          // submit
          console.log("openCustDialog res", res);
          nextTick(() => {
            resolve(res.value[0]);
          });
        },
        onCancel: (error: any) => {
          // cancel 回调
          console.log("openCustDialog error", error);
        }
      };
      state.MattressDialogRef.show(_params, label);
    });
  };

  return {
    ...toRefs(state),
    columns,
    columns_detail,
    columns_yw,
    columns_qd,
    RetriveMattressInterface,
    RefreshMattressInterfaceList,
    RefreshMattressInterfaceQdList,
    fModelChoseMattress
  };
};
